/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.
 */

#include "config/option/ratiorandomhostsoption.h"

#include <logfunc.hpp>

using namespace std;
using boost::program_options::value;
using boost::program_options::variables_map;
using I2n::Logger::GlobalLogger;

//-----------------------------------------------------------------------------
// RatioRandomHostsOption
//-----------------------------------------------------------------------------

RatioRandomHostsOption::RatioRandomHostsOption() :
    ConfigurationOption(
        "ratio-random-hosts",
        value<float>()->default_value( 1.0f ),
        "Ratio in (0,1] of hosts to use, picking them at random from host list"
    )
{
}

RatioRandomHostsOption::~RatioRandomHostsOption()
{
}

bool RatioRandomHostsOption::parse(
        const variables_map& vm,
        Configuration *configuration
)
{
    // default-source-network-interface
    if ( 1 <= vm.count( get_command_string() ) )
    {
        float ratio_random_hosts = vm[ get_command_string() ].as<float> ();
        configuration->set_ratio_random_hosts( ratio_random_hosts );

        GlobalLogger.info() << get_command_string() << "="
                << ratio_random_hosts << endl;

        return true;
    }

    return false;
}
