/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.

 Christian Herdtweck, Intra2net AG 2015
 */

#ifndef DNS_MASTER_H
#define DNS_MASTER_H

#include <map>
#include <pair>
#include <list>

#include <boost/smart_ptr.hpp>
#include <boost/noncopyable.hpp>
#include <boost/date_time/posix_time/posix_time_types.hpp>
#include <boost/asio/deadline_timer.hpp>
#include <boost/system/error_code.hpp>

#include "dns_neww/dnsresolver.h"

class DnsMaster;
typedef boost::smart_ptr<DnsMaster> DnsMasterItem;

typedef std::map<std::string, ResolverItem> resolver_map_type

typedef boost::posix_time::ptime timestamp_type;
typedef std::pair<HostAddress, timestamp_type> host_with_timestamp;
typedef std::vector<host_with_timestamp> host_ttl_time_list;
typedef std::map<std::string, host_ttl_time_list> cache_data_type;


class DnsMaster : boost::noncopyable
{
// the two functions called during init
public:
    static void create_master(const IoServiceItem &io_serv,
                              const boost::ip::address &name_server,
                              const std::string &cache_file);
    static ResolverItem get_resolver_for(const std::string &hostname);


// friendly functions in DnsResolver and their friends here
public:
    friend void DnsResolver::update_master(const AddressList &ips);
    friend std::string DnsResolver::get_next_ip();
    friend void cname_resolve_callback(const boost::system::error_code &error,
                                       const bool was_success,
                                       const int cname_count)
private:
    void update_ips(const AddressList &ips);
    host_ttl_time_list get_cached_results(std::string &hostname);


// implementation of singleton
private:
    static DnsMasterItem TheOnlyInstance;

    DnsMaster(const IoServiceItem &io_serv,
              const boost::ip::address &name_server,
              const std::string &cache_file);
public:
    ~DnsMaster();


// variables
private:
    IoServiceItem IoService;
    const boost::ip::address NameServer;
    std::string CacheFile;
    cache_data_type DataCache;
    resolver_map_type ResolverMap;
    boost::asio::deadline_timer SaveTimer;


// functions
private:
    void schedule_save(const boost::system::error_code &error);
    void save_to_cachefile() const;
    void load_from_cachefile();

};

#endif

// (created using vim -- the world's best text editor)

