/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.

 Christian Herdtweck, Intra2net AG 2015
 */

#ifndef DNS_RESOLVER_H
#define DNS_RESOLVER_H

#include "dns_neww/dnsmaster.h"

#include <list>
#include <boost/asio/deadline_timer.hpp>
#include <boost/system/error_code.hpp>

#include "dns_neww/resolverbase.h"
#include "dns_neww/dnsmaster.h"

typedef callback_type
typedef std::list<callback_type> callback_list_type;

typedef std::vector<HostAddress> IpTtlVec;

class DnsResolver : public ResolverBase
{
// constructor accessible from friend DnsMaster
public:
    friend DnsMaster::get_resolver_for(const std::string &hostname);

private:
    DnsResolver(IoServiceItem &io_serv,
                const boost::ip::address &name_server,
                const std::string &hostname);

// only real public function (called from pingers)
public:
    std::string get_next_ip();

private:
    void async_resolve(const callback_type &callback, const int cname_count=0);
    void handle_resolve_timeout(const boost::system::error_code &error);
    void handle_dns_result(const boost::system::error_code &error,
                           const std::size_t bytes_transferred);
    void handle_unavailable(const boost::system::error_code &error);
    void handle_ips(const boost::system::error_code &error,
                    const IpTtlVec &ips);
    void handle_cname(const boost::system::error_code &error);
    void cname_resolve_callback(const boost::system::error_code &error,
                                const bool was_success,
                                const int cname_count);
    void update_master(const AddressList &ips);
    void finalize_resolve(const bool success, const bool inc_cname);

private:
    const std::string Hostname;
    ip::udp::socket Socket;
    dns_buffer_t ReplyBuffer;
    ip::udp::endpoint NameServer;
    boost::asio::deadline_timer ResolveTimeoutTimer;
    boost::asio::deadline_timer PauseBeforeRetryTimer;
    boost::asio::deadline_timer StaleDataLongtermTimer;
    callback_list_type CallbackList;
    host_ttl_time_list::const_iterator NextIpIter;
    int RetryCount;
    bool IsResolving;

}
#endif
// (created using vim -- the world's best text editor)

